// JSONRequest.js
/*
 * Copyright 2009 Serhat Dirik

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
/*******************************************************************************
 * This class calls given JSON service .Requires XMLHTTPRequest.js loaded<br>
 * jsonrq = new JSONRequest('/YourWeb/JSONRR');<br>
 * cb=function() {<br>
 * print('Status :'+ this.status);<br>
 * print('Status :'+ this.statusText);<br>
 * print('-----------------------------');<br>
 * print(this.responseText);<br>
 * try{<br>
 * var res = jsonrq.resolveResult(this.responseText);<br>
 * print(res.list.join(','));<br>
 * }catch(e){<br>
 * if (e.message) alert(e.message) ; else alert(e);<br> } }<br>
 * jsonrq.call('softwareManagementSrv.getServices',[],cb,true);<br>
 * If you don't want to handle callback you can use it in simplest way jsonrq =
 * new JSONRequest('/WB_Manager/JSON-RPC'); 
 * result = jsonrq.call('softwareManagementSrv.getServices',[]); 
 * You must give at least
 * url parameter to constructor.If you give user and password parameters it uses
 * them while connecting to given url.
 * 
 * @author Serhat Dirik 
 * @version 1.0
 ******************************************************************************/

// The class gives you a cross browser XMLHttpRequest object.<br> 
var XMLHttpRequestFactory = {
  getXmlHttpRequest: function() {
    var _xmlhttp=null;
    if (window.XMLHttpRequest){	// code for Mozilla, etc.
			try{
				_xmlhttp=new XMLHttpRequest();
			}catch(e){
				txt = "An error has occured while creating XMLHttpRequest \n";
				if(e.message) txt += e.message ;
				else txt += e ;
				alert(txt);
				throw new Error(txt);
			}
		}else if (window.ActiveXObject){// code for IE
			try {
	        	_xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
	        	_isBrowserIE=true;
	      	} catch(e) {
	        	try {
	          		_xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
	          		_isBrowserIE=true;
	        	} catch(e) {
	          		txt = "An error has occured while creating Microsoft XMLHttpRequest \n";
					if(e.message) txt += e.message ;
					else txt += e ;
					alert(txt);
					throw new Error(txt);
	        	}
	        }	
		}else if(window.createRequest){
			try {
				_xmlhttp = window.createRequest();
			} catch (e) {
				txt = "An error has occured while creating window.createRequest \n";
				if(e.message) txt += e.message ;
				else txt += e ;
				alert(txt);
				throw new Error(txt); 
			}

		}else{
			txt = "Your browser doesn't support XMLHttpSupport";
			alert(txt);
			throw new Error(txt);
		}
	 return _xmlhttp;
  }
}


function JSONRequest(url, user, password) {
	if (!url || url == null) {
		throw new Error("Invalid URL specified for JSONRequest");
	}
	var _url = url;
	var _user = user;
	var _password = password;
	var _XMLHttpRequestPool = new Array();

	function _resolveResult(resultText) {
		var obj;
		try {
			eval("obj = " + resultText);
		} catch (e) {
			txt = "An error occured while parsing result \n";
			if (e.message)
				txt += e.message;
			else
				txt += e;
			throw new Error(txt);
		}
		if(obj.error){
			txt = " Code:" + obj.error.errorCode + ":" + obj.error.errorMsg
			+ " \n";

			var err = new Error(txt);
			err.errorCode = obj.error.errorCode;
			err.errorMsg = obj.error.errorMsg;
			err.errorName = obj.error.errorName;
			throw err;
		}
		return obj;

	}
	this.resolveResult = _resolveResult;

	function _getXMLHTTPRequest() {
		var _xmlhttp;
		for (_xmlhttp in _XMLHttpRequestPool) {
			if (_xmlhttp.readyState == 0 || _xmlhttp.readyState == 4)
				return _xmlhttp;
		}
		_xmlhttp = XMLHttpRequestFactory.getXmlHttpRequest();
		_XMLHttpRequestPool[_XMLHttpRequestPool.length] = _xmlhttp;
		return _xmlhttp;
	}

	// escape a character
	function _escapeJSONChar(c) {
		if (c == "\"" || c == "\\")
			return "\\" + c;
		else if (c == "\b")
			return "\\b";
		else if (c == "\f")
			return "\\f";
		else if (c == "\n")
			return "\\n";
		else if (c == "\r")
			return "\\r";
		else if (c == "\t")
			return "\\t";
		var hex = c.charCodeAt(0).toString(16);
		if (hex.length == 1)
			return "\\u000" + hex;
		else if (hex.length == 2)
			return "\\u00" + hex;
		else if (hex.length == 3)
			return "\\u0" + hex;
		else
			return "\\u" + hex;
	}
	// encode a string into JSON format
	function _escapeJSONString(s) {
		// The following should suffice but Safari's regex is b0rken
		// (doesn't support callback substitutions)
		//
		// return "\"" + s.replace(/([^\u0020-\u007f]|[\\\"])/g,
		// escapeJSONChar) + "\"";
		// Rather inefficient way to do it
		var parts = s.split("");
		for ( var i = 0; i < parts.length; i++) {
			var c = parts[i];
			if (c == '"' || c == '\\' || c.charCodeAt(0) < 32
					|| c.charCodeAt(0) >= 128)
				parts[i] = _escapeJSONChar(parts[i]);
		}
		return "\"" + parts.join("") + "\"";
	}

	// Marshall objects to JSON format
	function _toJSON(o) {
		if (o == null) {
			return 'null';
		} else if (o.constructor == String) {
			return _escapeJSONString(o);
		} else if (o.constructor == Number) {
			return o.toString();
		} else if (o.constructor == Boolean) {
			return o.toString();
		} else if (o.constructor == Date) {
			return '{classHintAttribute: "java.util.Date", time: ' + o
					.valueOf() + '}';
		} else if (o.constructor == Array) {
			var v = [];
			for ( var i = 0; i < o.length; i++)
				v.push(_toJSON(o[i]));
			return "[" + v.join(", ") + "]";
		} else {
			var v = [];
			for (attr in o) {
				if (o[attr] == null)
					v.push("\"" + attr + "\": null");
				else if (typeof o[attr] == "function")
					; // skip
				else
					v.push(_escapeJSONString(attr) + ": " + _toJSON(o[attr]));
			}
			return "{" + v.join(", ") + "}";
		}
	}

	function _send(id, method, parameters, headers, cb) {
		if (!method || method == null) {
			throw new Error('method is not specified!');
		}
		if (!parameters) {
			throw new Error('parameters is not specified!');
		}
		if (cb && typeof cb != 'function') {
			throw new Error('Call Back must be a function!');
		}
		
		var asynch=true;
		if (arguments.length < 5 ||!cb || cb == null){
			asynch = false;
		} 
		
		var _xmlhttp = _getXMLHTTPRequest();
		var obj = {};
		obj.id = id;
		obj.method = method;
		obj.header = headers;
		obj.params = parameters;
		var data = _toJSON(obj);
		_xmlhttp.open("POST", _url, asynch, _user, _password);
		// setRequestHeader is missing in Opera 8 Beta
		// try { xmlhttp.open.setRequestHeader("Content-type", "text/plain"); }
		// catch(e) {}
//		xmlhttp.channel.notificationCallbacks = {
//			getInterface : function(iid, instance) {
//				Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
//				return null;
//			}
//		};
		if (cb) {
			_xmlhttp.onreadystatechange = function() {
				if (_xmlhttp.readyState == 4) {
					/*var res = new Object();
					res.status = xmlhttp.status;
					res.statusText = xmlhttp.statusText;
					res.responseText = xmlhttp.responseText;
					res.responseXML = xmlhttp.responseXML;
					res.cb = cb;*/
					cb(_xmlhttp.responseText);
				}
			}
		} else {
			_xmlhttp.onreadystatechange = function() {
			};
		}
		_xmlhttp.send(data);
		if (!cb) {
			var result = null;
			if (_xmlhttp.status == 200) {
				try {
					result = this.resolveResult(_xmlhttp.responseText);
				} catch (e) {
					//if (e.message) alert(e.message) ; else alert(e);
					throw e;
				}
			} else {
				//alert(xmlhttp.status + " " + xmlhttp.statusText);
				throw new Error(_xmlhttp.status + " " + _xmlhttp.statusText);
			}
			return result;
		}
	}

	this.send = _send;

	this.toJSON = _toJSON;
}

function getContextPath() {
	if (window.contextPath && window.contextPath != null) {
		return window.contextPath;
	} else {
		var path = location.pathname;
		var arr = path.split("/");
		var context = "";
		if (arr.length >= 2) {
			context = arr[1];
		}
		return "/" + context;
	}
}
